@@ -3,8 +3,8 @@
 ### Free software under the terms of the GNU General Public License, version 2,
 ### a copy of which is available at http://www.r-project.org/Licenses/.
 ###
-### Copyright (C) 2013-2014 Sebastian Meyer
-### Time-stamp: <[polyCub.iso.R] by SM Mit 07/05/2014 10:29 (CEST)>
+### Copyright (C) 2013-2015 Sebastian Meyer
+### Time-stamp: <[polyCub.iso.R] 2015-02-16 12:18 (CET) by SM>
 ################################################################################
 
 
@@ -19,10 +19,10 @@
 #' function as argument \code{intrfr}.
 #' The two-dimensional integration problem thereby reduces to an efficient
 #' adaptive quadrature in one dimension.
-#' See Meyer and Held (2014, Section 2.4 of Supplement B) for mathematical
+#' See Meyer and Held (2014, Supplement B, Section 2.4) for mathematical
 #' details.
 #'
-#' @inheritParams polyCub.SV
+#' @inheritParams plotpolyf
 #' @param intrfr analytical antiderivative of \eqn{r f_r(r)} from 0 to \code{R}
 #' (first argument, not necessarily named \code{"R"}, must be vectorized).
 #' If missing, \code{intrfr} is approximated numerically using
@@ -37,6 +37,9 @@
 #' to be specified. If \code{TRUE}, the set of test
 #' \code{r}'s defaults to a \code{\link{seq}} of length 20 from 1 to
 #' the maximum absolute x or y coordinate of any edge of the \code{polyregion}.
+#' @param plot logical indicating if an image of the function should be plotted
+#' together with the polygonal domain, i.e.,
+#' \code{\link{plotpolyf}(polyregion, f, \dots)}.
 #' @return The approximate integral of the isotropic function
 #' \code{f} over \code{polyregion}.\cr
 #' If the \code{intrfr} function is provided (which is assumed to be exact), an
@@ -55,10 +58,9 @@
 #'
 #' Meyer, S. and Held, L. (2014).
 #' Power-law models for infectious disease spread.
-#' \emph{Annals of Applied Statistics}. In press, available as
-#' \href{http://arxiv.org/abs/1308.5115}{arXiv:1308.5115}.
-#' Supplements are available from
-#' \url{http://www.biostat.uzh.ch/research/manuscripts/powerlaw.html}.
+#' \emph{The Annals of Applied Statistics}, \bold{8} (3), 1612-1639.\cr
+#' DOI-Link: \url{http://dx.doi.org/10.1214/14-AOAS743},
+#' \href{http://arxiv.org/abs/1308.5115}{arXiv:1308.5115}
 #' @keywords math spatial
 #' @family polyCub-methods
 #' @example inst/examples/polyCub.iso.R
@@ -72,13 +74,17 @@ polyCub.iso <- function (polyregion, f, intrfr, ..., center,
                                 # which means anticlockwise vertex order with
                                 # first vertex not repeated
     getError <- !missing(intrfr) # can't estimate error of double approximation
+    center <- as.vector(center, mode = "numeric")
+    stopifnot(length(center) == 2L, is.finite(center))
 
     ## check 'intrfr' function
     rs <- if (isTRUE(check.intrfr)) {
         seq(1, max(abs(unlist(lapply(polys, "[", c("x","y"))))), length.out=20L)
     } else if (identical(check.intrfr, FALSE)) {
         numeric(0L)
-    } else check.intrfr
+    } else {
+        check.intrfr
+    }
     intrfr <- checkintrfr(intrfr, f, ..., center=center, control=control, rs=rs)
 
     ## plot polygon and function image
@@ -115,12 +121,13 @@ checkintrfr <- function (intrfr, f, ..., center, control = list(),
     if (!missing(f)) {
         f <- match.fun(f)
         rfr <- function (r, ...)
-            r * f(cbind(center[1]+r, center[2], deparse.level=0), ...)
-        quadrfr1 <- function (r, ...) integrate(rfr, 0, r, ...)$value
+            r * f(cbind(center[1L]+r, center[2L], deparse.level=0L), ...)
+        quadrfr1 <- function (R, ...) integrate(rfr, 0, R, ...)$value
         if (length(control))
-            body(quadrfr1)[[2]] <- as.call(c(as.list(body(quadrfr1)[[2]]),
+            body(quadrfr1)[[2L]] <- as.call(c(as.list(body(quadrfr1)[[2L]]),
                                              control))
-        quadrfr <- function (r, ...) sapply(r, quadrfr1, ..., USE.NAMES=FALSE)
+        quadrfr <- function (R, ...)
+            vapply(X = R, FUN = quadrfr1, FUN.VALUE = 0, ..., USE.NAMES = FALSE)
         if (missing(intrfr)) {
             return(quadrfr)
         } else if (doCheck) {
@@ -135,8 +142,9 @@ checkintrfr <- function (intrfr, f, ..., center, control = list(),
                 warning("'intrfr' might be incorrect: ", comp)
             } else cat("OK\n")
         }
-    } else if (doCheck)
+    } else if (doCheck) {
         stop("numerical verification of 'intrfr' requires 'f'")
+    }
     
     match.fun(intrfr)
 }
@@ -155,22 +163,25 @@ checkintrfr <- function (intrfr, f, ..., center, control = list(),
                    intrfr, ..., center=center,
                    control=control, .witherror=.witherror)
     if (.witherror) {
-        structure(sum(sapply(ints, "[", 1, simplify=TRUE, USE.NAMES=FALSE)),
-                  abs.error=sum(sapply(ints, "[", 2,
-                  simplify=TRUE, USE.NAMES=FALSE)))
-    } else sum(unlist(ints, recursive=FALSE, use.names=FALSE))
+        res <- sum(vapply(X=ints, FUN="[", FUN.VALUE=0, 1L, USE.NAMES=FALSE))
+        attr(res, "abs.error") <-
+            sum(vapply(X=ints, FUN="[", FUN.VALUE=0, 2L, USE.NAMES=FALSE))
+        res
+    } else {
+        sum(unlist(ints, recursive=FALSE, use.names=FALSE))
+    }
 }
 
 ## cubature method for a single polygon
 polyCub1.iso <- function (poly, intrfr, ..., center,
                           control = list(), .witherror = TRUE)
 {
-    xy <- cbind(poly[["x"]], poly[["y"]], deparse.level=0)
+    xy <- cbind(poly[["x"]], poly[["y"]], deparse.level=0L)
     nedges <- nrow(xy)
     intedges <- erredges <- numeric(nedges)
     for (i in seq_len(nedges)) {
-        v0 <- xy[i,] - center
-        v1 <- xy[if (i==nedges) 1L else i+1L,] - center
+        v0 <- xy[i, ] - center
+        v1 <- xy[if (i==nedges) 1L else i+1L, ] - center
         int <- lineInt(v0, v1, intrfr, ..., control=control)
         intedges[i] <- int$value
         erredges[i] <- int$abs.error
@@ -180,7 +191,11 @@ polyCub1.iso <- function (poly, intrfr, ..., center,
     ##     if ((1 - 2 * as.numeric(poly$hole)) * sign(int) == -1)
     ##         warning("wrong sign if positive integral")
     ## }
-    c(int, if (.witherror) sum(erredges))
+    if (.witherror) {
+        c(int, sum(erredges))
+    } else {
+        int
+    }
 }
 
 ## line integral for one edge
@@ -188,19 +203,24 @@ polyCub1.iso <- function (poly, intrfr, ..., center,
 lineInt <- function (v0, v1, intrfr, ..., control = list())
 {
     d <- v1 - v0
-    num <- v1[2]*v0[1] - v1[1]*v0[2]  # = d[2]*p[,1] - d[1]*p[,2]
-                                      # for any point p on the edge
+    num <- v1[2L]*v0[1L] - v1[1L]*v0[2L]  # = d[2]*p[,1] - d[1]*p[,2]
+                                          # for any point p on the edge
+    if (num == 0) { # i.e., if 'center' is part of this polygon edge
+        return(list(value = 0, abs.error = 0))
+    }
     integrand <- function (t) {
         ## get the points on the edge corresponding to t
-        p <- cbind(v0[1] + t*d[1], v0[2] + t*d[2], deparse.level=0)
-        norm2 <- .rowSums(p^2, length(t), 2)
+        p <- cbind(v0[1L] + t*d[1L], v0[2L] + t*d[2L], deparse.level=0L)
+        norm2 <- .rowSums(p^2, length(t), 2L)
         ints <- intrfr(sqrt(norm2), ...)
         ##ints[is.infinite(ints)] <- 1e300
         num * ints / norm2
     }
     if (length(control)) {              # use slower do.call()-construct
         do.call("integrate", c(list(integrand, 0, 1), control))
-    } else integrate(integrand, 0, 1)
+    } else {
+        integrate(integrand, 0, 1)
+    }
 }
 
 ## equally fast method _only_ for convex polygonal domains including the origin
